#include "DataFormats/VertexReco/interface/Vertex.h" 
#include "FWCore/MessageLogger/interface/MessageLogger.h"

using namespace reco;
using namespace std;

Vertex::Vertex () {}

Vertex::Vertex ( const Point & p, const CovarianceMatrix & c, double chi2, double ndf,
                       size_t size ) :
  chi2_(chi2), ndof_(ndf), p_(p), c_(c), size_(size)
{}

float Vertex::trackWeight ( const TrackRef & track ) const
{
  trackRef_iterator it = find(tracks_begin(), tracks_end(), track);                                                                            
  if (it==tracks_end()) return 0.0;                                                                                                            
  size_t pos = it - tracks_begin();                                                                                                            
  return weights_[pos];
  // edm::LogError("Vertex") << "track weight is currently always 1";
  // return 1.;
}

const Vertex::Point & Vertex::position() const
{
  return p_;
}

double Vertex::x() const
{
  return p_.x();
}

double Vertex::y() const
{
  return p_.y();
}

double Vertex::z() const
{
  return p_.z();
}

Vertex::CovarianceMatrix Vertex::covariance() const
{
  return c_;
}

double Vertex::chi2() const
{
  return chi2_;
}

double Vertex::ndof() const
{
  return ndof_;
}

void Vertex::removeTracks()
{
  // does nothing, we dont keep tracks
}

size_t Vertex::tracksSize() const
{
  return size_;
}

track_iterator Vertex::tracks_begin() const
{
  return tracks_.begin();
}

void Vertex::add ( const TrackRef & r, double w )
{
  tracks_.push_back ( r );
  weights_.push_back ( w );
}

void Vertex::add ( const TrackRef & r, const Track & refit, double w )
{
  tracks_.push_back ( r );
  refittedTracks_.push_back ( refit );
  weights_.push_back ( w );
}


track_iterator Vertex::tracks_end() const
{
  return tracks_.end();
}
