#include "RaveBase/RaveInterface/rave/Charge.h"
#include "RaveBase/RaveInterface/rave/Vector3D.h"
#include "RaveBase/RaveInterface/rave/Vector6D.h"
#include "RaveBase/RaveInterface/rave/Covariance6D.h"
#include "RaveBase/RaveInterface/rave/PerigeeMomentum3D.h"
#include "RaveBase/RaveInterface/rave/PerigeeState5D.h"
#include "RaveBase/RaveInterface/rave/PerigeeCovariance5D.h"

#ifdef WITH_KINEMATICS
#include "RaveBase/RaveInterface/rave/Vector7D.h"
#include "RaveBase/RaveInterface/rave/Covariance7D.h"
#include "RaveBase/RaveInterface/rave/PerigeeState6D.h"
#include "RaveBase/RaveInterface/rave/PerigeeCovariance6D.h"
#endif

namespace rave {

/**
 * @class RaveToPerigeeObjects converts from euclidian space to perigee coords.
 *
 * This conversion class is used internally by several data classes to provide
 * their data in perigee parametrization.
 *
 * @author Fabian Moser <fabiamos@gmail.com>
 */

class RaveToPerigeeObjects
{
  public:
    PerigeeMomentum3D convert( const Vector3D &, const Charge ) const;
    PerigeeState5D convert( const Vector6D &, const Charge ) const;
    PerigeeCovariance5D convert( const Covariance6D & error, const Vector6D & state, const Charge charge ) const;

#ifdef WITH_KINEMATICS
  public:
    PerigeeState6D convert( const Vector7D &, const Charge ) const;
    PerigeeCovariance6D convert( const Covariance7D & error, const Vector7D & state, const Charge charge ) const;
#endif
};

} // namespace rave
