#include "RaveBase/Converters/interface/PerigeeToRaveObjects.h"

#include "RaveBase/Converters/interface/RaveToCmsObjects.h"
#include "RaveBase/Converters/interface/CmsToRaveObjects.h"
#include "RaveBase/Converters/interface/RaveToPerigeeObjects.h"
#include "FWCore/MessageLogger/interface/MessageLogger.h"
#include "RaveTools/Converters/interface/MagneticFieldSingleton.h"

#include "boost/cast.hpp"

namespace rave {

Vector3D PerigeeToRaveObjects::convert(const PerigeeMomentum3D &) const
{
  // TODO
  edm::LogError("PerigeeToRaveObjects" ) << "Missing implementation!";
  exit(0);
}

Vector6D PerigeeToRaveObjects::convert(
    const PerigeeState5D & state, const Charge charge, 
    const Point3D & referencePoint) const
{
  RaveToCmsObjects frameworkInputConverter;
  PerigeeTrajectoryParameters ptp = frameworkInputConverter.convert( state, charge );
  GlobalPoint ref = frameworkInputConverter.convert( referencePoint );

  // We need the charge knowledge to know from the transverseCurvature what pt is.
  float pt;
  float field = MagneticFieldSingleton::Instance()->inInverseGeV( ref ).z();
  float signTC = boost::numeric_cast<float>( - charge );
  bool isCharged = ( charge != 0 );
  if ( isCharged ) {
    pt = field / ptp.transverseCurvature()*signTC;
  } else {
    pt = 1. / ptp.transverseCurvature();
  }

  PerigeeConversions parameterConverter;
  GlobalPoint  position = parameterConverter.positionFromPerigee( ptp, ref );
  GlobalVector momentum = parameterConverter.momentumFromPerigee( ptp, pt, ref );

  CmsToRaveObjects frameworkOutputConverter;
  return Vector6D( frameworkOutputConverter.convert( position ),
                   frameworkOutputConverter.convert( momentum ) );
}

Covariance6D PerigeeToRaveObjects::convert(
    const PerigeeCovariance5D & error, const Vector6D & state,
    const Charge charge) const
{
  PerigeeState5D ps = RaveToPerigeeObjects().convert( state, charge );

  RaveToCmsObjects frameworkInputConverter;
  GlobalTrajectoryParameters gtp =
      frameworkInputConverter.convert( state, charge );
  PerigeeTrajectoryError pte =
      frameworkInputConverter.convert( error, ps );

  PerigeeConversions parameterConverter;
  CurvilinearTrajectoryError cte =
      parameterConverter.curvilinearError(pte, gtp);

  // The FreeTrajectoryState is used for conversion curvilinear -> cartesian
  CmsToRaveObjects frameworkOutputConverter;
  return frameworkOutputConverter.convert(
             FreeTrajectoryState( gtp, cte ).cartesianError() );
}

}
