#ifndef _RavePerigeeState5D_H_
#define _RavePerigeeState5D_H_

#include <rave/PerigeeMomentum3D.h>

#include "boost/operators.hpp"

namespace rave
{

/**
 * @class PerigeeState5D stores track perigee parameters
 *
 * This data class manages a track state in his perigee representation.
 *
 * The choice of parameters matches the L3 convention also used in LCIO files
 * see T.Kraemer LC-DET-2006-004
 */

class RaveDllExport PerigeeState5D :
    public rave::PerigeeMomentum3D,
    boost::equality_comparable< PerigeeState5D >
{
  public:
    PerigeeState5D ( float omega, float tanLambda, float phi, float d0, 
                     float z0 );
    PerigeeState5D();

    float d0() const;
    float z0() const;

    bool equal ( const PerigeeState5D & other ) const;

  private:
    float theD0;
    float theZ0;
};

RaveDllExport bool operator== ( const rave::PerigeeState5D & lhs, const rave::PerigeeState5D & rhs );

} // namespace rave

#endif

