//
// C++ Interface: PerigeeState6D
//
// Author: Fabian Moser <fabiamos@gmail.com>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef RAVEPERIGEESTATE6D_H
#define RAVEPERIGEESTATE6D_H

#include <rave/PerigeeState5D.h>

namespace rave
{

/**
 * @class PerigeeState6D perigee parameter with invariant mass
 *
 * Extends the usual 5-parameter perigee perameter set for the invariant mass.
 *
 * @author Fabian Moser <fabiamos@gmail.com>
 */

class RaveDllExport PerigeeState6D :
      public PerigeeState5D,
      boost::equality_comparable< PerigeeState5D >
{
  public:
    PerigeeState6D();
    PerigeeState6D ( double omega, double tanLambda, double phi, double d0,
                     double z0, double mass );

    ~PerigeeState6D();

    double mass() const; //!< returns the invariant mass

    bool equal ( const PerigeeState6D& other ) const;

  private:
    double theMass;

};

RaveDllExport bool operator== ( const rave::PerigeeState5D & lhs, const rave::PerigeeState5D & rhs );

}

#endif
