#include "TrackingTools/TransientTrack/interface/TransientTrackFromFTS.h"
#include "TrackingTools/PatternTools/interface/TransverseImpactPointExtrapolator.h"
#include "FWCore/MessageLogger/interface/MessageLogger.h"
#include <iostream>

using namespace reco;

TransientTrackFromFTS::TransientTrackFromFTS() : 
  theField(0), initialTSOSAvailable(false), initialTSCPAvailable(false),
  trackAvailable(false)
{}

TransientTrackFromFTS::TransientTrackFromFTS(const FreeTrajectoryState & fts) :
  initialFTS(fts), theField(&(initialFTS.parameters().magneticField())),
  initialTSOSAvailable(false), initialTSCPAvailable(false), trackAvailable(false)
{}

TransientTrackFromFTS::TransientTrackFromFTS( const TransientTrackFromFTS & tt ) :
  initialFTS(tt.initialFreeState()), theField(tt.field()), initialTSOSAvailable(false),
  initialTSCPAvailable(false), trackAvailable(false)
{
  if (tt.initialTSOSAvailable) {
    initialTSOS= tt.impactPointState();
    initialTSOSAvailable = true;
  }
  if (tt.initialTSCPAvailable) {
    initialTSCP= tt.impactPointTSCP();
    initialTSCPAvailable = true;
  }
}

TrajectoryStateOnSurface TransientTrackFromFTS::impactPointState() const
{
  if (!initialTSOSAvailable) calculateTSOSAtVertex();
  return initialTSOS;
}

TrajectoryStateClosestToPoint TransientTrackFromFTS::impactPointTSCP() const
{
  if (!initialTSCPAvailable) {
    initialTSCP = builder(initialFTS, initialFTS.position());
    initialTSCPAvailable = true;
  }
  return initialTSCP;
}

TrajectoryStateOnSurface TransientTrackFromFTS::outermostMeasurementState() const
{
  throw cms::Exception("LogicError") << 
    "TransientTrack built from a FreeTrajectoryState (TransientTrackFromFTS) can not have an outermostMeasurementState";
}

TrajectoryStateOnSurface TransientTrackFromFTS::innermostMeasurementState() const
{
  throw cms::Exception("LogicError") << 
    "TransientTrack built from a FreeTrajectoryState (TransientTrackFromFTS) can not have an innermostMeasurementState";
}

void TransientTrackFromFTS::calculateTSOSAtVertex() const
{
  TransverseImpactPointExtrapolator tipe(theField);
  initialTSOS = tipe.extrapolate(initialFTS, initialFTS.position());
  initialTSOSAvailable = true;
}

TrajectoryStateOnSurface 
TransientTrackFromFTS::stateOnSurface(const GlobalPoint & point) const
{
  TransverseImpactPointExtrapolator tipe(theField);
  return tipe.extrapolate(initialFTS, point);
}

