
#include <iostream>
#include <sstream>

#include <rave/Version.h>
#include <rave/TransientTrackKinematicParticle.h>
#include <rave/KinematicConstraintBuilder.h>
#include <rave/KinematicTreeFactory.h>

namespace {
  std::vector< rave::KinematicParticle > createParticles()
  {
    rave::Vector7D state1 ( 0.0001, 0.0001, 0.0001, -31.2685, 13.0785, 28.7524, 0.1057 );
    rave::Covariance7D cov1 (
         1.5e-7,    3.6e-7,    4.0e-14,
                    8.5e-7,    9.6e-14,
                               1.7e-6,
                                        -1.4e-16,  -3.4e-16,   1.8e-24,
                                        -3.3e-16,  -8.1e-16,   4.3e-24,
                                        -3.9e-9,   -9.4e-9,    5.0e-17,
                                         4.9e-3,   -2.0e-3,   -4.4e-3,
                                                    9.2e-4,    1.8e-3,
                                                               4.1e-3,
           0,         0,         0,        0,         0,         0,      6.2 );
    rave::TransientTrackKinematicParticle particle1 (state1, cov1, +1.0, 100, 100);

    rave::Vector7D state2 (-0.0006, -0.0006, 0.0018 , -57.1634, -57.6416, -40.0142 , 0.1057 );
    rave::Covariance7D cov2 (
        5.0e-7,    -5.0e-7,   -1.1e-14,
                    5.0e-7,    1.1e-14,
                               1.2e-6,
                                         1.5e-16,  -1.5e-16,   3.4e-24,
                                        -1.5e-16,   1.5e-16,  -3.4e-24,
                                         4.2e-9,   -4.2e-9,    9.7e-17,
                                         6.7e-2,    6.7e-2,    4.7e-2,
                                                    6.8e-2,    4.7e-2,
                                                               3.3e-2,
           0,         0,         0,        0,         0,         0,      6.2 );
    rave::TransientTrackKinematicParticle particle2 (state2, cov2, -1.0, 100, 100);

    std::vector< rave::KinematicParticle > particles;
    particles.push_back( particle1 );
    particles.push_back( particle2 );

    return particles;
  }

  std::string fit ()
  {
    std::ostringstream o;
    rave::ConstantMagneticField mfield(0.,0.,4.);
    rave::KinematicTreeFactory factory ( mfield, rave::VacuumPropagator() );
    rave::KinematicConstraint constraint =
        rave::KinematicConstraintBuilder().createTwoTrackMassKinematicConstraint( 91.187 );
    std::vector < rave::KinematicParticle > input_particles = createParticles();
    
	rave::KinematicTree tree_kalman;
    try {
      tree_kalman = factory.useVertexFitter( input_particles );
    } catch ( ... ) {};
    if (!tree_kalman.isValid())
    {
      o << "The unconstrained decay could not be reconstructed. ";
    }
    else
    {
      rave::KinematicParticle topParticle = tree_kalman.topParticle();
      o << "The unconstrained mother particle is " << topParticle.fullstate() << ". ";
    }

	rave::KinematicTree tree_lagrange;
	try {
      tree_lagrange = factory.useVertexFitter( input_particles, constraint );
    } catch ( ... ) {};
    if (!tree_lagrange.isValid())
    {
      o << "The constrained decay could not be reconstructed.";
    }
    else
    {
      rave::KinematicParticle topParticle = tree_lagrange.topParticle();
      o << "The constrained mother particle is " << topParticle.fullstate() << ".";
    }

    return o.str();
  }

  std::string version()
  {
    std::ostringstream o;
    o << "Rave Version " << rave::Version();
    return o.str();
  }
}

int main(void)
{
  std::cout << "This is Rave Version " << rave::Version() << std::endl;
  std::cout << "Fitting says: " << fit() << std::endl;
  return 0;
}

